<?php namespace Myriad\Installations\Controllers;

use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Myriad\Installations\Exceptions\InstallationValidationException;
use Myriad\Installations\InstallationManager;
use Myriad\Installations\Models\Installation;
use Myriad\Installations\Validation\IInstallationValidationProvider;

class InstallationsController extends Controller {
  /**
   * @var InstallationManager
   */
  private $installationManager;
  /**
   * @var IInstallationValidationProvider
   */
  private $installationValidationProvider;

  /**
   * InstallationsController constructor.
   *
   * @param InstallationManager $installationManager
   * @param IInstallationValidationProvider $installationValidationProvider
   */
  public function __construct(InstallationManager $installationManager, IInstallationValidationProvider $installationValidationProvider) {
    $this->installationManager = $installationManager;
    $this->installationValidationProvider = $installationValidationProvider;
  }

  /**
   * Find an Installation by id
   *
   * @param $uuid
   * @param Request $request
   * @return Response
   */
  public function show($uuid, Request $request) {
    try {
      $this->installationValidationProvider->validateUUID($uuid);
    } catch (InstallationValidationException $e) {
      return $this->renderException($request, $e);
    }

    $installation = $this->installationManager->findInstallationByUUID($uuid);

    if (!$installation) {
      $mnfe = new ModelNotFoundException();
      $mnfe->setModel(Installation::class);
      return $this->renderException($request, $mnfe);
    }

    return $this->renderShow($request, $installation);
  }

  /**
   * Create a new Installation
   *
   * @param Request $request
   * @return Response
   */
  public function store($uuid, Request $request) {
    try {
      $this->installationValidationProvider->validateUUID($uuid);
    } catch (InstallationValidationException $e) {
      return $this->renderException($request, $e);
    }

    $data = array_merge($this->installationManager->getParamsFromRequest(), $request->all());
    $data['uuid'] = $uuid;
    
    $validator = $this->installationValidationProvider->getValidator($data);
    if ($validator->fails()) {
      return $this->renderException($request, new InstallationValidationException($validator));
    }

    $installation = $this->installationManager->createOrUpdateInstallation($uuid, $data);

    return $this->renderShow($request, $installation);
  }

  /**
   * Used to render successful responses from the show() method.
   *
   * @param Request $request
   * @param Installation $installation
   * @return JsonResponse
   */
  public function renderShow(Request $request, Installation $installation) {
    return new JsonResponse(['data' => $installation]);
  }

  /**
   * Used to render successful responses from the show() method.
   *
   * @param Request $request
   * @param Installation $installation
   * @return JsonResponse
   */
  public function renderStore(Request $request, Installation $installation) {
    return new JsonResponse(['data' => $installation]);
  }

  /**
   * @param Request $request
   * @param \Exception $exception
   * @return null
   * @throws \Exception
   */
  public function renderException(Request $request, \Exception $exception) {
    throw $exception;
  }

}