<?php namespace Myriad\Installations;

use Illuminate\Contracts\Config\Repository;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Myriad\Installations\Models\Installation;
use Myriad\Installations\Validation\IInstallationValidationProvider;

class InstallationManager {

  /**
   * @var Request
   */
  private $request;

  /**
   * @var Repository
   */
  private $config;

  /**
   * @var Collection[Installation] Cache of installations
   */
  private $installations;

  /**
   * @var IInstallationValidationProvider
   */
  private $validator;

  /**
   * InstallationManager constructor.
   *
   * @param Request $request
   * @param Repository $config
   * @param IInstallationValidationProvider $validator
   */
  public function __construct(Request $request, Repository $config, IInstallationValidationProvider $validator) {
    $this->request = $request;
    $this->config = $config;
    $this->validator = $validator;
    $this->installations = collect();
  }

  /**
   * @return Installation|null get the current installation
   */
  public function getInstallation() {
    $uuid = $this->getUuidFromRequest();
    if (!$this->installations->has($uuid)) {
      $this->createOrUpdateInstallation($uuid);
    }
    return $this->installations->get($uuid);
  }

  /**
   * Find an installation by its id
   *
   * @param $id int installation id
   * @return Installation|null
   */
  public function findInstallationById($id) {
    $installation = $this->installations->keyBy('id')->get($id);
    if (!$installation) {
      $installation = Installation::query()->find($id);
      if ($installation) {
        $this->installations->put($installation->uuid, $installation);
      }
    }
    return $installation;
  }

  /**
   * Find an installation by its uuid
   *
   * @param $uuid string installation uuid
   * @return Installation|null
   */
  public function findInstallationByUUID($uuid) {
    $uuid = $this->validator->normalizeUUID($uuid);
    $installation = $this->installations->get($uuid);
    if (!$installation) {
      $installation = Installation::query()->where('uuid', $uuid)->first();
      if ($installation) {
        $this->installations->put($uuid, $installation);
      }
    }
    return $installation;
  }

  /**
   * Get the UUID from the http request
   *
   * @return string
   */
  public function getUuidFromRequest() {
    $uuid = $this->request->header($this->config->get('installations.request_map.app_installation_id.header'),
                                  $this->request->query($this->config->get('installations.request_map.app_installation_id.query')));
    return $this->validator->normalizeUUID($uuid);
  }

  /**
   * Get the app name from the http request
   *
   * @return string
   */
  public function getAppNameFromRequest() {
    return $this->request->header($this->config->get('installations.request_map.app_name.header'),
                                  $this->request->query($this->config->get('installations.request_map.app_name.query')));
  }

  /**
   * Get the app version from the http request
   *
   * @return string
   */
  public function getAppVersionFromRequest() {
    return $this->request->header($this->config->get('installations.request_map.app_version.header'),
                                  $this->request->query($this->config->get('installations.request_map.app_version.query')));
  }

  /**
   * Get the user agent from the http request
   *
   * @return array|string
   */
  public function getUserAgentFromRequest() {
    return $this->request->server('HTTP_USER_AGENT');
  }

  /**
   * Get the default attributes from the request
   *
   * @return array
   */
  public function getParamsFromRequest() {
    return [
      'uuid' => $this->getUuidFromRequest(),
      'app_name' => $this->getAppNameFromRequest(),
      'app_version' => $this->getAppVersionFromRequest(),
      'user_agent' => $this->getUserAgentFromRequest(),
    ];
  }

  /**
   * Create or update an installation for the given $uuid and request
   *
   * @param $uuid string the installation uuid
   * @param array $params
   * @return Installation the installation
   */
  public function createOrUpdateInstallation($uuid, $params = []) {
    $uuid = $this->validator->normalizeUUID($uuid);
    $params['uuid'] = $uuid;

    $installation = $this->findInstallationByUUID($uuid);
    if (!$installation) {
      $installation = new Installation();
    }

    $params = array_merge($this->getParamsFromRequest(), $params);
    $params = Arr::only($params, ['uuid', 'app_name', 'app_version', 'platform_name', 'platform_version', 'locale', 'timezone', 'push_type', 'push_id']);
    
    $installation->fill(array_merge($this->getParamsFromRequest(), $params));
    $installation->save();

    $this->installations->put($uuid, $installation);

    return $installation;
  }

  /**
   * Get the validation provider
   *
   * @return IInstallationValidationProvider
   */
  public function getValidator() {
    return $this->validator;
  }

}